# -*- coding: utf-8 -*-
"""
Módulo que contiene clases para analizar un fichero con una lista
de emisiones de CO2 por países

@author: To-Do
@date:   jun 2019
"""

# imports
# To-Do

class Emision:
    """
    Contiene un dato de emisión de un país en un año concreto

    Attributes:
        __anyo: año en el que se mide la emisión
        __pais: nombre del país
        __continente: nombre del continente
        __emision: valor de emisión de CO2 en toneladas métricas por habitante
                   en el año almacenado
    """

    def __init__(self, anyo: int, pais: str, continente: str, emision: float):
        """
        Constructor que recibe como parámetros los valores iniciales de los
        atributos

        Params:
        anyo: año en el que se mide la emisión
        pais: nombre del país
        continente: nombre del continente
        emision: valor de emisión de CO2 en toneladas métricas por habitante
                 en el año indicado
        """
        self.__anyo = anyo
        self.__pais = pais
        self.__continente = continente
        self.__emision = emision

    def get_anyo(self)-> int:
        """
        Retorna el año del dato de emisión
        """
        return self.__anyo

    def get_pais(self)-> str:
        """
        Retorna el país del dato de emisión
        """
        return self.__pais

    def get_continente(self)-> str:
        """
        Retorna el continente del dato de emisión
        """
        return self.__continente

    def get_emision(self)-> float:
        """
        Retorna el dato de emisión de CO2 en toneladas métricas por habitante
        """
        return self.__emision

class FicheroNoExiste(Exception):
    """
    Indica que el fichero que se quiere leer no existe
    """

class PaisIncorrecto(Exception):
    """
    Indica que un país que se ha especificado tiene un nombre incorrecto
    """

class ListaEmisiones:
    """
    La clase dispone entre otros atributos de una lista de objetos de la
    clase Emision y diversos métodos para analizar las emisiones

    Attributes:
        __lista: una lista de objetos de la clase Emision
        __nombre_fichero: el nombre del fichero del que se leen los datos
        __fuente: fuente de los datos
    """

    def __init__(self, nombre_fichero: str):
        """
        Crea la lista de emisiones vacía y rellena los atributos
        nombre_fichero y fuente

        Args:
            nombre_fichero: el nombre del fichero donde se encuentran
                            los datos de emisiones
        Raises:
            FicheroNoExiste: si el fichero especificado no existe
        """
        # To-Do

    def lee(self):
        """
        Lee los datos de emisiones del fichero cuyo nombre se indica
        en el argumento nombre_fichero y los mete en la lista.
        El fichero contiene tres líneas de encabezamiento y luego un
        dato de emisión por cada línea, con sus campos (año, país,
        continente y emisión) separados por comas.
        Se supone que los datos del fichero son correctos.

        Si el fichero no existe este método no hace nada
        """
        try:
            with open(self.__nombre_fichero, 'r') as fichero:
                # Saltamos las líneas de cabecera
                fichero.readline()
                fichero.readline()
                fichero.readline()

                # Leemos el resto de las líneas
                for linea in fichero:
                    datos: List[str] = linea.split(",")
                    emi = Emision(int(datos[0]), datos[1], datos[2],
                                  float(datos[3]))
                    self.__lista.append(emi)
        except IOError:
            pass

    def resumen(self):
        """
        Muestra en pantalla un resumen de los datos de la lista.
        """
        # Escribe el encabezamiento
        print("Emisiones de C02 per cápita, en toneladas",
              f"Número de datos {len(self.__lista)}")
        # Escribe el nombre del fichero y la fuente
        print(f"Nombre del fichero {self.__nombre_fichero}.",
              f"Fuente de los datos: {self.__fuente}")
        print(f"{'Año':4} {'País':35}{'Continente':14}{'Emision':7}")
        # Escribe los 3 primeros datos, uno por línea
        print("Tres primeros datos:")
        for emi in self.__lista[0:3]:
            print(f"{emi.get_anyo():4} {emi.get_pais():35}"+
                  f"{emi.get_continente():14}{emi.get_emision():7.3f}")
        # Escribe los 3 últimos datos, uno por línea
        print("Tres últimos datos:")
        for emi in self.__lista[-3:]:
            print(f"{emi.get_anyo():4} {emi.get_pais():35}"+
                  f"{emi.get_continente():14}{emi.get_emision():7.3f}")

    def emisiones_de_pais(self, pais: str)-> float:
        """
        Calcula y retorna la media de las emisiones del país que se pasa
        como parámetro para todos los años en los que se tienen datos
        de ese país

        Args:
            pais: el pais buscado
        Raises:
            PaisIncorrecto: si el pais no se encuentra en la lista
        """
        # To-Do

    def emision_media_superior(self, valor: float)-> str:
        """
        Calcula y retorna el primer país para el que la media de sus
        emisiones en los años para los que se tienen datos supera
        el valor indicado por el parámetro. Si no se encuentra el pais,
        se retorna None
        """
        # Primero se meten en un conjunto los nombres de todos
        # los países de la lista.
        # To-Do

        # Se busca en ese conjunto el primer país para el que la media
        # de sus emisiones en los años para los que se tienen datos
        # supera el valor indicado por el parámetro.
        # Si se encuentra, se retorna el nombre de ese país.
        # Si no se encuentra, se retorna None.
        # To-Do

def main():
    """
    Prueba la clase ListaEmisiones
    """
    # To-Do

if __name__ == '__main__':
    main()
